#!/usr/bin/env python
# Stegano - Stegano is a pure Python steganography module.
# Copyright (C) 2010-2026 Cédric Bonhomme - https://www.cedricbonhomme.org
#
# For more information : https://github.com/cedricbonhomme/Stegano
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = "Cedric Bonhomme"
__version__ = "$Revision: 0.7 $"
__date__ = "$Date: 2016/08/25 $"
__license__ = "GPLv3"

import argparse

from PIL import Image

try:
    from stegano.steganalysis import parity
except Exception:
    print("Install Stegano: pipx install Stegano")


def main():
    parser = argparse.ArgumentParser(prog="stegano-steganalysis-parity")
    parser.add_argument(
        "-i",
        "--input",
        dest="input_image_file",
        required=True,
        help="Input image file.",
    )
    parser.add_argument(
        "-o",
        "--output",
        dest="output_image_file",
        required=True,
        help="Output image file.",
    )
    arguments = parser.parse_args()

    input_image_file = Image.open(arguments.input_image_file)
    output_image = parity.steganalyse(input_image_file)
    output_image.save(arguments.output_image_file)



#!/usr/bin/env python
# Stegano - Stegano is a pure Python steganography module.
# Copyright (C) 2010-2026 Cédric Bonhomme - https://www.cedricbonhomme.org
#
# For more information : https://github.com/cedricbonhomme/Stegano
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = "Cedric Bonhomme"
__version__ = "$Revision: 0.7 $"
__date__ = "$Date: 2016/03/18 $"
__revision__ = "$Date: 2019/06/04 $"
__license__ = "GPLv3"

try:
    from stegano import wav
except Exception:
    print("Install stegano: pipx install Stegano")

import argparse

from stegano import tools


def main():
    parser = argparse.ArgumentParser(prog="stegano-lsb")
    subparsers = parser.add_subparsers(
        help="sub-command help", dest="command", required=True
    )

    # Subparser: Hide
    parser_hide = subparsers.add_parser("hide", help="hide help")
    # Original audio
    parser_hide.add_argument(
        "-i",
        "--input",
        dest="input_audio_file",
        required=True,
        help="Input audio file.",
    )
    parser_hide.add_argument(
        "-e",
        "--encoding",
        dest="encoding",
        choices=tools.ENCODINGS.keys(),
        default="UTF-8",
        help="Specify the encoding of the message to hide."
        " UTF-8 (default) or UTF-32LE.",
    )

    group_secret = parser_hide.add_mutually_exclusive_group(required=True)
    # Non binary secret message to hide
    group_secret.add_argument(
        "-m", dest="secret_message", help="Your secret message to hide (non binary)."
    )
    # Binary secret message to hide
    group_secret.add_argument(
        "-f", dest="secret_file", help="Your secret to hide (Text or any binary file)."
    )

    # Audio containing the secret
    parser_hide.add_argument(
        "-o",
        "--output",
        dest="output_audio_file",
        required=True,
        help="Output audio containing the secret.",
    )

    # Subparser: Reveal
    parser_reveal = subparsers.add_parser("reveal", help="reveal help")
    parser_reveal.add_argument(
        "-i",
        "--input",
        dest="input_audio_file",
        required=True,
        help="Input audio file.",
    )
    parser_reveal.add_argument(
        "-e",
        "--encoding",
        dest="encoding",
        choices=tools.ENCODINGS.keys(),
        default="UTF-8",
        help="Specify the encoding of the message to reveal."
        " UTF-8 (default) or UTF-32LE.",
    )

    arguments = parser.parse_args()

    if arguments.command == "hide":
        if arguments.secret_message is not None:
            secret = arguments.secret_message
        elif arguments.secret_file != "":
            secret = tools.binary2base64(arguments.secret_file)

        wav.hide(
            input_file=arguments.input_audio_file,
            message=secret,
            encoding=arguments.encoding,
            output_file=arguments.output_audio_file,
        )

    elif arguments.command == "reveal":
        try:
            secret = wav.reveal(
                input_file=arguments.input_audio_file, encoding=arguments.encoding
            )
        except IndexError:
            print("Impossible to detect message.")
            exit(0)
        if arguments.secret_binary is not None:
            data = tools.base642binary(secret)
            with open(arguments.secret_binary, "wb") as f:
                f.write(data)
        else:
            print(secret)



#!/usr/bin/env python
# Stegano - Stegano is a pure Python steganography module.
# Copyright (C) 2010-2026 Cédric Bonhomme - https://www.cedricbonhomme.org
#
# For more information : https://github.com/cedricbonhomme/Stegano
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = "Cedric Bonhomme"
__version__ = "$Revision: 0.2.2 $"
__date__ = "$Date: 2016/05/26 $"
__revision__ = "$Date: 2017/01/18 $"
__license__ = "GPLv3"

import piexif

from stegano import tools


def hide(
    input_image_file,
    img_enc,
    secret_message=None,
    secret_file=None,
    img_format=None,
):
    """Hide a message (string) in an image."""
    from base64 import b64encode
    from zlib import compress

    if secret_file is not None:
        with open(secret_file, "rb") as f:
            secret_message = f.read()

    try:
        text = compress(b64encode(bytes(secret_message, "utf-8")))
    except Exception:
        text = compress(b64encode(secret_message))

    img = tools.open_image(input_image_file)

    if img_format is None:
        img_format = img.format

    if "exif" in img.info:
        exif_dict = piexif.load(img.info["exif"])
    else:
        exif_dict = {}
        exif_dict["0th"] = {}
    exif_dict["0th"][piexif.ImageIFD.ImageDescription] = text
    exif_bytes = piexif.dump(exif_dict)
    img.save(img_enc, format=img_format, exif=exif_bytes)
    img.close()
    return img


def reveal(input_image_file):
    """Find a message in an image."""
    from base64 import b64decode
    from zlib import decompress

    img = tools.open_image(input_image_file)

    try:
        if img.format in ["JPEG", "TIFF"]:
            if "exif" in img.info:
                exif_dict = piexif.load(img.info.get("exif", b""))
                description_key = piexif.ImageIFD.ImageDescription
                encoded_message = exif_dict["0th"][description_key]
            else:
                encoded_message = b""
        else:
            raise ValueError("Given file is neither JPEG nor TIFF.")
    finally:
        img.close()

    return b64decode(decompress(encoded_message))



#!/usr/bin/env python
# Stegano - Stegano is a pure Python steganography module.
# Copyright (C) 2010-2026 Cédric Bonhomme - https://www.cedricbonhomme.org
#
# For more information : https://github.com/cedricbonhomme/Stegano
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = "Cedric Bonhomme"
__version__ = "$Revision: 0.1 $"
__date__ = "$Date: 2017/02/06 $"
__license__ = "GPLv3"

import argparse

try:
    from stegano import exifHeader
except Exception:
    print("Install stegano: sudo pip install Stegano")


def main():
    parser = argparse.ArgumentParser(prog="stegano-exif")
    subparsers = parser.add_subparsers(help="sub-command help", dest="command")

    parser_hide = subparsers.add_parser("hide", help="hide help")
    parser_hide.add_argument(
        "-i", "--input", dest="input_image_file", help="Input image file."
    )
    parser_hide.add_argument(
        "-m", dest="secret_message", help="Your raw secret message to hide."
    )
    parser_hide.add_argument(
        "-o", "--output", dest="output_image_file", help="Output image file."
    )
    parser_hide.add_argument(
        "-f", "--secret-file", dest="secret_file", help="Your secret file to hide."
    )

    parser_reveal = subparsers.add_parser("reveal", help="reveal help")
    parser_reveal.add_argument(
        "-i", "--input", dest="input_image_file", help="Input image file."
    )

    arguments = parser.parse_args()

    if arguments.command == "hide" and arguments.secret_message:
        secret = exifHeader.hide(arguments.input_image_file, arguments.secret_message)
        secret.save(arguments.output_image_file)

    elif arguments.command == "hide" and arguments.secret_file:
        secret = exifHeader.hide(arguments.input_image_file, arguments.secret_file)
        secret.save(arguments.output_image_file)

    elif arguments.command == "reveal":
        secret = exifHeader.reveal(arguments.input_image_file)
        print(secret)



#!/usr/bin/env python
# Stegano - Stéganô is a basic Python Steganography module.
# Copyright (C) 2010-2026  Cédric Bonhomme - https://www.cedricbonhomme.org
#
# For more information : https://github.com/cedricbonhomme/Stegano
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = "Cedric Bonhomme"
__version__ = "$Revision: 0.2 $"
__date__ = "$Date: 2010/10/01 $"
__revision__ = "$Date: 2017/02/06 $"
__license__ = "GPLv3"

import wave
from typing import IO, Union

from stegano import tools


def hide(
    input_file: Union[str, IO[bytes]],
    message: str,
    output_file: Union[str, IO[bytes]],
    encoding: str = "UTF-8",
):
    """
    Hide a message (string) in a .wav audio file.

    Use the lsb of each PCM encoded sample to hide the message string characters as ASCII values.
    The first eight bits are used for message_length of the string.
    """
    message_length = len(message)
    assert message_length != 0, "message message_length is zero"
    assert message_length < 255, "message is too long"

    output = wave.open(output_file, "wb")
    with wave.open(input_file, "rb") as input:
        # get .wav params
        nchannels, sampwidth, framerate, nframes, comptype, _ = input.getparams()
        assert comptype == "NONE", "only uncompressed files are supported"

        nsamples = nframes * nchannels

        message_bits = f"{message_length:08b}" + "".join(
            tools.a2bits_list(message, encoding)
        )
        assert len(message_bits) <= nsamples, "message is too long"

        # copy over .wav params to output
        output.setnchannels(nchannels)
        output.setsampwidth(sampwidth)
        output.setframerate(framerate)

        # encode message in frames
        frames = bytearray(input.readframes(nsamples))
        for i in range(nsamples):
            if i < len(message_bits):
                if message_bits[i] == "0":
                    frames[i] = frames[i] & ~1
                else:
                    frames[i] = frames[i] | 1

        # write out
        output.writeframes(frames)


def reveal(input_file: Union[str, IO[bytes]], encoding: str = "UTF-8"):
    """
    Find a message in an image.

    Check the lsb of each PCM encoded sample for hidden message characters (ASCII values).
    The first eight bits are used for message_length of the string.
    """
    message = ""
    encoding_len = tools.ENCODINGS[encoding]
    with wave.open(input_file, "rb") as input:
        nchannels, _, _, nframes, comptype, _ = input.getparams()
        assert comptype == "NONE", "only uncompressed files are supported"

        nsamples = nframes * nchannels
        frames = bytearray(input.readframes(nsamples))

        # Read first 8 bits for message length
        length_bits = ""
        for i in range(8):
            length_bits += str(frames[i] & 1)
        message_length = int(length_bits, 2)

        # Read message bits
        message_bits = ""
        for i in range(8, 8 + message_length * encoding_len):
            message_bits += str(frames[i] & 1)

        # Convert bits to string
        chars = [
            chr(int(message_bits[i : i + encoding_len], 2))
            for i in range(0, len(message_bits), encoding_len)
        ]
        message = "".join(chars)
    return message



#!/usr/bin/env python
# Stegano - Stéganô is a basic Python Steganography module.
# Copyright (C) 2010-2026  Cédric Bonhomme - https://www.cedricbonhomme.org
#
# For more information : https://github.com/cedricbonhomme/Stegano
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

__author__ = "Cedric Bonhomme"
__version__ = "$Revision: 0.2 $"
__date__ = "$Date: 2010/10/01 $"
__revision__ = "$Date: 2017/02/06 $"
__license__ = "GPLv3"

from typing import IO, Union, cast

from stegano import tools


def hide(input_image: Union[str, IO[bytes]], message: str):
    """
    Hide a message (string) in an image.

    Use the red portion of a pixel (r, g, b) tuple to
    hide the message string characters as ASCII values.
    The red value of the first pixel is used for message_length of the string.
    """
    message_length = len(message)
    assert message_length != 0, "message message_length is zero"
    assert message_length < 255, "message is too long"
    img = tools.open_image(input_image)
    # Ensure image mode is RGB
    if img.mode != "RGB":
        img = img.convert("RGB")
    # Use a copy of image to hide the text in
    encoded = img.copy()
    width, height = img.size
    index = 0
    for row in range(height):
        for col in range(width):
            pixel = cast(tuple[int, int, int], img.getpixel((col, row)))
            r, g, b = pixel
            # first value is message_length of message
            if row == 0 and col == 0 and index < message_length:
                asc = message_length
            elif index <= message_length:
                c = message[index - 1]
                asc = ord(c)
            else:
                asc = r
            encoded.putpixel((col, row), (asc, g, b))
            index += 1
    img.close()
    return encoded


def reveal(input_image: Union[str, IO[bytes]]):
    """
    Find a message in an image.

    Check the red portion of an pixel (r, g, b) tuple for
    hidden message characters (ASCII values).
    The red value of the first pixel is used for message_length of string.

    Raises:
        ValueError: If the message length from the image exceeds the maximum
            allowed length (254) or exceeds the available pixels in the image.
    """
    img = tools.open_image(input_image)
    try:
        # Ensure image mode is RGB
        if img.mode != "RGB":
            img = img.convert("RGB")
        width, height = img.size
        max_possible_length = (width * height) - 1
        message = ""
        index = 0
        message_length = 0

        for row in range(height):
            for col in range(width):
                pixel = cast(tuple[int, int, int], img.getpixel((col, row)))
                r, g, b = pixel
                # First pixel r value is length of message
                if row == 0 and col == 0:
                    message_length = r
                    # Validate message length to prevent malicious input
                    if message_length > 254:
                        raise ValueError(
                            f"Invalid message length: {message_length} exceeds maximum (254)"
                        )
                    if message_length > max_possible_length:
                        raise ValueError(
                            f"Invalid message length: {message_length} exceeds "
                            f"available pixels ({max_possible_length})"
                        )
                elif index <= message_length:
                    message += chr(r)
                index += 1
    finally:
        img.close()
    return message